@@ -9,7 +9,7 @@ import { ICodeMirror, CodeMirrorEditor } from '@jupyterlab/codemirror';
 import { ISettingRegistry } from '@jupyterlab/settingregistry';
 import { IDisposable } from '@lumino/disposable';
 
-import { VimCellManager } from './codemirrorCommands';
+import { VimCellManager, IKeybinding } from './codemirrorCommands';
 import { addJLabCommands } from './labCommands';
 
 const PLUGIN_NAME = '@axlair/jupyterlab_vim';
@@ -42,15 +42,23 @@ async function activateCellVim(
     },
     isToggled: () => enabled
   });
+
+  const userKeybindings = ((
+    await settingRegistry.get(`${PLUGIN_NAME}:plugin`, 'extraKeybindings')
+  ).composite as unknown) as Array<IKeybinding>;
+
   // eslint-disable-next-line prettier/prettier
   const globalCodeMirror = jlabCodeMirror.CodeMirror as unknown as CodeMirrorEditor;
   let cellManager: VimCellManager | null = null;
   let escBinding: IDisposable | null = null;
-
-  let addedCommands: Array<IDisposable> | null = null;
   let hasEverBeenEnabled = false;
 
-  cellManager = new VimCellManager(app.commands, globalCodeMirror, enabled);
+  cellManager = new VimCellManager({
+    commands: app.commands,
+    cm: globalCodeMirror,
+    enabled,
+    userKeybindings
+  });
   // it's ok to connect here because we will never reach the vim section unless
   // ensureVimKeyMap has been called due to the checks for enabled.
   // we need to have now in order to keep track of the last active cell
@@ -59,13 +67,21 @@ async function activateCellVim(
     cellManager.onActiveCellChanged,
     cellManager
   );
+
+  addJLabCommands(app, tracker, globalCodeMirror);
+
   async function updateSettings(
     settings: ISettingRegistry.ISettings
   ): Promise<void> {
+    const userKeybindings = ((
+      await settingRegistry.get(`${PLUGIN_NAME}:plugin`, 'extraKeybindings')
+    ).composite as unknown) as Array<IKeybinding>;
+
     enabled = settings.get('enabled').composite === true;
     app.commands.notifyCommandChanged(TOGGLE_ID);
     if (cellManager) {
       cellManager.enabled = enabled;
+      cellManager.userKeybindings = userKeybindings;
     }
     if (enabled) {
       escBinding?.dispose();
@@ -74,24 +90,19 @@ async function activateCellVim(
         await app.restored;
         await jlabCodeMirror.ensureVimKeymap();
       }
-      addedCommands = addJLabCommands(app, tracker, globalCodeMirror);
-      cellManager?.modifyCell(cellManager.lastActiveCell);
-      tracker.forEach(notebook => {
-        notebook.node.dataset.jpVimMode = 'true';
-      });
     } else {
-      addedCommands?.forEach(command => command.dispose());
       escBinding = app.commands.addKeyBinding({
         command: 'notebook:enter-command-mode',
         keys: ['Escape'],
         selector: '.jp-Notebook.jp-mod-editMode'
       });
-      cellManager?.modifyCell(cellManager.lastActiveCell);
-      tracker.forEach(notebook => {
-        notebook.node.dataset.jpVimMode = 'false';
-      });
     }
 
+    tracker.forEach(notebook => {
+      notebook.node.dataset.jpVimMode = `${enabled}`;
+    });
+    cellManager?.modifyCell(cellManager.lastActiveCell);
+
     // make sure our css selector is added to new notebooks
     tracker.widgetAdded.connect((sender, notebook) => {
       notebook.node.dataset.jpVimMode = `${enabled}`;
